// Licensed to Elasticsearch B.V. under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Elasticsearch B.V. licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package linux

import (
	"bufio"
	"bytes"
	"fmt"
	"os"
)

const procOneCgroup = "/proc/1/cgroup"

// IsContainerized returns true if this process is containerized.
func IsContainerized() (bool, error) {
	data, err := os.ReadFile(procOneCgroup)
	if err != nil {
		if os.IsNotExist(err) {
			return false, nil
		}

		return false, fmt.Errorf("failed to read process cgroups: %w", err)
	}

	return isContainerizedCgroup(data)
}

func isContainerizedCgroup(data []byte) (bool, error) {
	s := bufio.NewScanner(bytes.NewReader(data))
	for n := 0; s.Scan(); n++ {
		line := s.Bytes()

		// Following a suggestion on Stack Overflow on how to detect
		// being inside a container: https://stackoverflow.com/a/20012536/235203
		if bytes.Contains(line, []byte("docker")) || bytes.Contains(line, []byte(".slice")) || bytes.Contains(line, []byte("lxc")) || bytes.Contains(line, []byte("kubepods")) {
			return true, nil
		}
	}

	return false, s.Err()
}
