/****************************************************************************
** Meta object code from reading C++ file 'User.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../internal/frontend/bridge-gui/bridgepp/bridgepp/User/User.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'User.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8bridgepp10EUserStateE_t {};
} // unnamed namespace

template <> constexpr inline auto bridgepp::EUserState::qt_create_metaobjectdata<qt_meta_tag_ZN8bridgepp10EUserStateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bridgepp::EUserState",
        "State",
        "SignedOut",
        "Locked",
        "Connected"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(1, 1, QMC::EnumIsScoped).add({
            {    2, State::SignedOut },
            {    3, State::Locked },
            {    4, State::Connected },
        }),
    };
    return QtMocHelpers::metaObjectData<EUserState, qt_meta_tag_ZN8bridgepp10EUserStateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject bridgepp::EUserState::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp10EUserStateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp10EUserStateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8bridgepp10EUserStateE_t>.metaTypes,
    nullptr
} };

void bridgepp::EUserState::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EUserState *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *bridgepp::EUserState::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bridgepp::EUserState::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp10EUserStateE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int bridgepp::EUserState::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8bridgepp4UserE_t {};
} // unnamed namespace

template <> constexpr inline auto bridgepp::User::qt_create_metaobjectdata<qt_meta_tag_ZN8bridgepp4UserE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "bridgepp::User",
        "toggleSplitModeForUser",
        "",
        "userID",
        "makeItActive",
        "logoutUser",
        "removeUser",
        "configureAppleMailForUser",
        "address",
        "idChanged",
        "id",
        "usernameChanged",
        "username",
        "passwordChanged",
        "addressesChanged",
        "avatarTextChanged",
        "avatarText",
        "loggedInChanged",
        "loggedIn",
        "stateChanged",
        "UserState",
        "state",
        "splitModeChanged",
        "splitMode",
        "usedBytesChanged",
        "byteCount",
        "totalBytesChanged",
        "toggleSplitModeFinished",
        "isSyncingChanged",
        "syncing",
        "syncProgressChanged",
        "syncProgress",
        "toggleSplitMode",
        "logout",
        "remove",
        "configureAppleMail",
        "emitToggleSplitModeFinished",
        "primaryEmailOrUsername",
        "password",
        "addresses",
        "usedBytes",
        "totalBytes",
        "isSyncing"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'toggleSplitModeForUser'
        QtMocHelpers::SignalData<void(QString const &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Signal 'logoutUser'
        QtMocHelpers::SignalData<void(QString const &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'removeUser'
        QtMocHelpers::SignalData<void(QString const &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'configureAppleMailForUser'
        QtMocHelpers::SignalData<void(QString const &, QString const &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'idChanged'
        QtMocHelpers::SignalData<void(QString const &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'usernameChanged'
        QtMocHelpers::SignalData<void(QString const &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'passwordChanged'
        QtMocHelpers::SignalData<void(QString const &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'addressesChanged'
        QtMocHelpers::SignalData<void(QStringList const &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 2 },
        }}),
        // Signal 'avatarTextChanged'
        QtMocHelpers::SignalData<void(QString const &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'loggedInChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(UserState)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'splitModeChanged'
        QtMocHelpers::SignalData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Signal 'usedBytesChanged'
        QtMocHelpers::SignalData<void(float)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 25 },
        }}),
        // Signal 'totalBytesChanged'
        QtMocHelpers::SignalData<void(float)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 25 },
        }}),
        // Signal 'toggleSplitModeFinished'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isSyncingChanged'
        QtMocHelpers::SignalData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Signal 'syncProgressChanged'
        QtMocHelpers::SignalData<void(float)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 31 },
        }}),
        // Slot 'toggleSplitMode'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'logout'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'remove'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'configureAppleMail'
        QtMocHelpers::SlotData<void(QString const &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'emitToggleSplitModeFinished'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'primaryEmailOrUsername'
        QtMocHelpers::MethodData<QString() const>(37, 2, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'id'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'username'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'password'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'addresses'
        QtMocHelpers::PropertyData<QStringList>(39, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'avatarText'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'state'
        QtMocHelpers::PropertyData<UserState>(21, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'splitMode'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'usedBytes'
        QtMocHelpers::PropertyData<float>(40, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'totalBytes'
        QtMocHelpers::PropertyData<float>(41, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'isSyncing'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'syncProgress'
        QtMocHelpers::PropertyData<float>(31, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<User, qt_meta_tag_ZN8bridgepp4UserE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject bridgepp::User::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp4UserE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp4UserE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8bridgepp4UserE_t>.metaTypes,
    nullptr
} };

void bridgepp::User::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<User *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->toggleSplitModeForUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->logoutUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->removeUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->configureAppleMailForUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->idChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->usernameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->passwordChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->addressesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 8: _t->avatarTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->loggedInChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<UserState>>(_a[1]))); break;
        case 11: _t->splitModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->usedBytesChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 13: _t->totalBytesChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 14: _t->toggleSplitModeFinished(); break;
        case 15: _t->isSyncingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->syncProgressChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 17: _t->toggleSplitMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->logout(); break;
        case 19: _t->remove(); break;
        case 20: _t->configureAppleMail((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->emitToggleSplitModeFinished(); break;
        case 22: { QString _r = _t->primaryEmailOrUsername();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (User::*)(QString const & , bool )>(_a, &User::toggleSplitModeForUser, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(QString const & )>(_a, &User::logoutUser, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(QString const & )>(_a, &User::removeUser, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(QString const & , QString const & )>(_a, &User::configureAppleMailForUser, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(QString const & )>(_a, &User::idChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(QString const & )>(_a, &User::usernameChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(QString const & )>(_a, &User::passwordChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(QStringList const & )>(_a, &User::addressesChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(QString const & )>(_a, &User::avatarTextChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(bool )>(_a, &User::loggedInChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(UserState )>(_a, &User::stateChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(bool )>(_a, &User::splitModeChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(float )>(_a, &User::usedBytesChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(float )>(_a, &User::totalBytesChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)()>(_a, &User::toggleSplitModeFinished, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(bool )>(_a, &User::isSyncingChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (User::*)(float )>(_a, &User::syncProgressChanged, 16))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->username(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->password(); break;
        case 3: *reinterpret_cast<QStringList*>(_v) = _t->addresses(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->avatarText(); break;
        case 5: *reinterpret_cast<UserState*>(_v) = _t->state(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->splitMode(); break;
        case 7: *reinterpret_cast<float*>(_v) = _t->usedBytes(); break;
        case 8: *reinterpret_cast<float*>(_v) = _t->totalBytes(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isSyncing(); break;
        case 10: *reinterpret_cast<float*>(_v) = _t->syncProgress(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setID(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setUsername(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setPassword(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setAddresses(*reinterpret_cast<QStringList*>(_v)); break;
        case 4: _t->setAvatarText(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setState(*reinterpret_cast<UserState*>(_v)); break;
        case 6: _t->setSplitMode(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setUsedBytes(*reinterpret_cast<float*>(_v)); break;
        case 8: _t->setTotalBytes(*reinterpret_cast<float*>(_v)); break;
        case 9: _t->setIsSyncing(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setSyncProgress(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *bridgepp::User::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *bridgepp::User::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8bridgepp4UserE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int bridgepp::User::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void bridgepp::User::toggleSplitModeForUser(QString const & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void bridgepp::User::logoutUser(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void bridgepp::User::removeUser(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void bridgepp::User::configureAppleMailForUser(QString const & _t1, QString const & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void bridgepp::User::idChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void bridgepp::User::usernameChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void bridgepp::User::passwordChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void bridgepp::User::addressesChanged(QStringList const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void bridgepp::User::avatarTextChanged(QString const & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void bridgepp::User::loggedInChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void bridgepp::User::stateChanged(UserState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void bridgepp::User::splitModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void bridgepp::User::usedBytesChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void bridgepp::User::totalBytesChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void bridgepp::User::toggleSplitModeFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void bridgepp::User::isSyncingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void bridgepp::User::syncProgressChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}
QT_WARNING_POP
